#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::OS::linux - Linux specific configuration information
#
package Security::OS::linux;

use Security::util::stat;
use Security::util::users;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check );

our (%perms, %files, %pkgs);
our $rpm = '/bin/rpm';
our $verifyargs = '--verify --all --nomd5 --nomtime --nosignature --nodeps --nodigest --noscripts --nosize';

sub init {
	print ( STDERR  "In Security::OS::linux::init()\n") if ( $main::opts{'d'} );

	$main::showmount = '/usr/sbin/showmount';
	$main::exportsfile = '/etc/exports';
	$main::securefile='/etc/securetty';
	$main::kerneldir='/lib/modules';
	$main::sudofile = '/etc/sudoers';
}

sub check {
	print ( STDERR  "In Security::OS::linux::check()\n") if ( $main::opts{'d'} );

	&Security::rootlogin::check_securettys() 
		if ( defined &Security::rootlogin::check_securettys );

	&check_osfiles() if ( -f $rpm );
}

sub check_osfiles {
	print ( STDERR  "In Security::OS::linux::check_osfiles()\n") if ( $main::opts{'d'} );

	_get_errors();
	_find_pkg_info();

	# Now go through and tell us what is wrong....
	_save_errors();
}

sub print {
	print ( STDERR  "In Security::OS::linux::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking rpm check command output:\n", %perms);
}

sub _get_errors {
	open (FP, "$rpm $verifyargs|") || &main::security_die ("Could not run $rpm $verifyargs\n");
	while (<FP>) {
		chomp;
		s/ \(Permission denied\)$//;
		my ($fmode, $type, $file) = split;
		$file = $type if ( ! defined $file );

		if ( -e $file || $file =~ m|^/| ) {
			if ( $fmode ne 'missing' ) {
				my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = &Security::util::stat::get_stat($file, 'lstat');
				my $owner => &Security::util::users::get('uid', $uid, 'name');
				my $group => scalar getgrgid($gid);
				$files{$file} = {
					size => $curbytes,
					mode => $mode,
					owner => $owner,
					group => $group,
					rdev => $rdev,
					failure => $fmode,
				}
			} else {
				$files{$file}->{'failure'} = $fmode;
			}
		} else {
			warn "Could not understand output ($_)\n";
		}
	}
	close FP;
}

sub _find_pkg_info {
	foreach my $file ( keys %files ) {
		my $pkg = `$rpm -qf --qf "%{NAME}" $file`;

		# Now get the pkg dump info.
		if ( defined $pkg ) {
			$files{$file}->{pkg} = $pkg;
			if ( ! exists $pkgs{$pkg} ) {
				open (FP, "$rpm -q --dump $pkg |") || &main::security_die ("Could not run $rpm -q --dump $pkg\n");
				while (<FP>) {
					chomp;
					my ($file, $size, $mtime, $md5, $mode, $owner, $group, $isconfig, $isdoc, $rdev, $symlink) = split;
					$pkgs{$pkg}->{$file} = {
						size => $size,
						mtime => $mtime,
						md5 => $md5,
						mode => oct($mode),
						owner => $owner,
						group => $group,
						rdev => $rdev,
						symlink => $symlink,
					}
				}
				close FP;
			}
		} else {
			&main::security_die ("Could not find package for file $file\n");
		}
	}
}

sub _save_errors {
	foreach my $file ( keys %files ) {
		my $pkg = $files{$file}->{pkg};
		if ( exists $pkgs{$pkg} ) {
			if ( $files{$file}->{failure} eq 'missing' ) {
				$perms{$file}->{'file type'} = '(' . &_format_type($pkgs{$pkg}->{$file}->{mode}) . ')';
				$perms{$file}->{'link'} = '-> ' . $pkgs{$pkg}->{$file}->{symlink}
					if (&S_ISLNK($pkgs{$pkg}->{$file}->{mode}));
				$perms{$file}->{'mode'} = '(' . &_format_mode($pkgs{$pkg}->{$file}->{mode}) . ')';
				$perms{$file}->{'owner'} = '(' . $pkgs{$pkg}->{$file}->{owner} . ')';
				$perms{$file}->{'group'} = '(' . $pkgs{$pkg}->{$file}->{group} . ')';
				$perms{$file}->{'missing'} = 1;
			} else {
				if ( $files{$file}->{failure} =~ /M/ ) {
					if (&S_IFMT($pkgs{$pkg}->{$file}->{mode}) != &S_IFMT($files{$file}->{mode})) {
						$perms{$file}->{'file type'} = '(' . &_format_type($pkgs{$pkg}->{$file}->{mode}) . ':' .  &_format_type($files{$file}->{mode}) . ')';
					} else {
						$perms{$file}->{'mode'} = '(' . &_format_mode($pkgs{$pkg}->{$file}->{mode}) . ':' .  &_format_mode($files{$file}->{mode}) . ')';
					}
				}
				if ( $files{$file}->{failure} =~ /D/ ) {
					$perms{$file}->{'device'} = '(' . $pkgs{$pkg}->{$file}->{rdev} . ':' .  $files{$file}->{rdev} . ')';
				}
				if ( $files{$file}->{failure} =~ /L/ ) {
					if ( $pkgs{$pkg}->{$file}->{symlink} eq 'X' ) {
						$perms{$file}->{'link'} = 'shold not be a link';
					} else {
						$perms{$file}->{'link'} = '-> ' . $pkgs{$pkg}->{$file}->{symlink};
					}
				}
				if ( $files{$file}->{failure} =~ /U/ ) {
					$perms{$file}->{'owner'} = '(' . $pkgs{$pkg}->{$file}->{owner} . ':' . $files{$file}->{owner} . ')';
				}
				if ( $files{$file}->{failure} =~ /G/ ) {
					$perms{$file}->{'group'} = '(' . $pkgs{$pkg}->{$file}->{group} . ':' . $files{$file}->{group} . ')';
				}
			}
		} else {
			&main::security_die ("Can't find package for file ($file)\n");
		}
	}
}

sub _format_type {
	my ($mode) = @_;
	return 'file' if (&S_ISREG($mode));
	return 'directory' if (&S_ISDIR($mode));
	return 'link' if (&S_ISLNK($mode));
	return 'socket' if (&S_ISSOCK($mode));
	return 'block' if (&S_ISBLK($mode));
	return 'character' if (&S_ISCHR($mode));
	return 'fifo' if (&S_ISFIFO($mode));
}

sub _format_mode {
	my ($mode) = @_;
	return sprintf "%4.4o", &S_IMODE($mode);
}

1;

__END__

=head1 NAME

Security::OS::linux

=head1 SYNOPSIS

Security::OS::linux::init();

Security::OS::linux::check();

=head1 DESCRIPTION

Sets variables for OS specific commands.

=head1 API

Defines the following API calls

=over 4

=item C<init()>

Init sets the following global variables:

=over 4

=item C<$main::showmount>

The location of the showmount command, used in the C<Security::exports> module.

=item C<$main::exportsfile>

The location of the exports file, used in C<Security::exports> module.

=item C<$main::sudofile>

The location of F<sudoers> file, used in C<Security::sudo> module.

=item C<$main::kerneldir>

The location of the kernel loadable modules, used in C<Security::kernel>
module.

=item C<$main::securefile>

The location of the list of ports root is allowed in on, used in
C<Security::rootlogin> module.

=back

=item C<check()>

Calls C<Security::rootlogin::check_securetty()> to check what root ports are
configured and compares it to what ports we know about.

=back

=cut

