#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::find - Filesystem traversal module
#
package Security::find;

use File::Find;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check() &add_wanted());

our ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes);
our (@funcs);

sub init {
	print ( STDERR  "In Security::find::init()\n") if ( $main::opts{'d'} );

	&main::read_config('ignoredir');

	# This is a hack since we want to prune the directories before
	# any other wanted routines are called.
	@funcs = ( \&prune_wanted );
	# &add_wanted(\&prune_wanted);
}

sub check {
	print ( STDERR "In Security::find::check()\n") if ( $main::opts{'d'} );

	find(\&_wanted, '/');
}

sub add_wanted {
	my ($func) = @_;
	print ( STDERR "In Security::find::add_wanted()\n") if ( $main::opts{'d'} );

	push @funcs, $func;
}

sub prune_wanted {
	my ($file) = @_;

	for (my ($i) = 0; $i <= $#main::ignoredir; $i++) {
		if ( $main::ignoredir[$i] eq $File::Find::name ) {
			$File::Find::prune = 1;
			printf STDERR "\tPruning Tree at $File::Find::name\n" 
				if ( $main::opts{'d'} );
			return;
		}
	} 
}

sub _wanted {
	for (my ($i) = 0; $i <= $#funcs; $i++) {
		&{$funcs[$i]} ($File::Find::name);
	}
}

1;

__END__

=head1 NAME

Security::find - File system traversal module

=head1 SYNOPSIS

Security::find::init()

Security::find::check()

Security::find::add_wanted(\&func)

=head1 DESCRIPTION

Module to traverse the filesystem, looking for files with specific
characteristics, as registered by other modules.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (ignoredir)> to get the list of directories to
prune out of our filesystem traversal.

Calls C<add_wanted (\&prune_wanted)> to implement the pruning of the
filesystem traversal.

=item C<check()>

Calls C<File::Find::find()> with the list of registered subroutines
to call for each file.

=back

It also provides the following API calls for other modules to use.

=over 4

=item C<add_wanted(\&func)>

A function to register new functions that should be called during the
filesystem traversal for each node.  Order is first come, first served.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignoredir>

The list of directories to prune out of the file system traversal.

=back

=head1 HISTORY

This idea of having only 1 file system traversal was suggeted in the
FreeBSD PR database.

=head1 SEE ALSO

L<File::Find>

L<find(1)>

=cut
