#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::passwd - missing or shared password control module
#
package Security::passwd;

use Security::util::users;
use Exporter;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT = qw( &init() &check() &print() );

our (%uid0, %nopass, %dupuid, %defaultpw, %duppasswd, %oldpass);

sub init {
	print ( STDERR  "In Security::passwd::init()\n") if ( $main::opts{'d'} );

	&main::read_config('ignorepasswd');
	&main::read_config('ignoreuid');
	&main::read_config('ignoredupuid');
	&main::read_config('ignoreduppasswd');
	&main::read_config('defaultpass');
}

sub check {
	print ( STDERR "In Security::password::check()\n") if ( $main::opts{'d'} );

	my ($oldname, $olduid);
	$olduid = -5;
	foreach my $name ( Security::util::users::get('name') ) {
		my $passwd = Security::util::users::get('name', $name, 'passwd');
		my $uid = Security::util::users::get('name', $name, 'uid');

		# Check to see who has uids of 0.  This comes from the security 
		# script from BSD 4.4.  We do not need to report the uid 0 on 
		# the 'root' account.
		$uid0{$name} = 1 
			if (! $uid && $name ne 'root' && 
				!grep (/^${name}$/, @main::ignoreuid));

		# Check to see who has accounts without passwords.  This comes 
		# from the security script from BSD 4.4.
		$nopass{$name} = 1 if ($passwd eq '' &&
				!grep (/^${name}$/, @main::ignorepasswd));

		# Check to see who has accounts with duplicate passwords.  
		if ( $passwd !~ /(^(x|NP|!+)?$)|\*/ ) {
			if ( exists $oldpass{$passwd} ) {
				$duppasswd{$name} = 'and ' . $oldpass{$passwd}
					if (!grep (/^${name}$/, @main::ignoreduppasswd));
				$oldpass{$passwd} .= ' ' . $name
			} else {
				$oldpass{$passwd} = $name;
			}
		}

		# Check to see who has accounts with default passwords.  The
		# admin will define the list of default encrypted passwords in
		# the config file.
		$defaultpw{$name} = 1 if (scalar @main::defaultpass &&
				grep (/^\Q${passwd}\E$/, @main::defaultpass));

		# Check to see what accounts have duplicate uid's (except the 
		# root account, which was handled earlier.  This idea comes from 
		# the HP-UX System Security manual
		$dupuid{$name} = $oldname if ( $olduid eq $uid && $uid &&
				!grep (/^${name}$/, @main::ignoredupuid));

		$olduid = $uid;
		$oldname = $name;
	}
}

sub print () {
	print ( STDERR  "In Security::passwd::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking for uids of 0:\n", %uid0);
	&main::print_data ("\n\nchecking for accounts with no passwords:\n", %nopass);
	&main::print_data ("\n\nchecking for accounts with default passwords:\n", %defaultpw);
	&main::print_data ("\n\nchecking for duplicate uids:\n", %dupuid);
	&main::print_data ("\n\nchecking for duplicate passwords:\n", %duppasswd);
}

1;

__END__

=head1 NAME

Security::passwd - missing or shared password control module

=head1 SYNOPSIS

Security::passwd::init()

Security::passwd::check()

Security::passwd::print()

=head1 DESCRIPTION

Looks for accounts that have no password set.

Looks for accounts that have the same password, as comparing the hash
values.

Looks for accounts that have a known password, as compared to a list of
configured hashes.

Looks for accounts with duplicate UIDs.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (ignorepasswd)> to get the list of accounts to ignore
that have no password.

Calls C<read_config (ignoreuid)> to get the list of accounts to ignore
that share uid 0.

Calls C<read_config (ignoredupuid)> to get the list of accounts to ignore
that share uids other than 0.

Calls C<read_config (ignoreduppasswd)> to get the list of accounts to ignore
that share passwords.

Calls C<read_config (defaultpass)> to get the list of password hashes that
we know of.

=item C<check()>

Traverses the accounts via C<Security::util::users::get()> looking for no
passwords, shared uids and passwords, and known passwords.

=item C<print()>

Calls C<print_data ()> to print out the list of accounts other than root
that have uid 0.

Calls C<print_data ()> to print out the list of accounts that do not have
a password set.

Calls C<print_data ()> to print out the list of accounts that have a known
password.

Calls C<print_data ()> to print out the list of accounts that have duplicate
uids.

Calls C<print_data ()> to print out the list of accounts that have duplicate
passwords.

=back 

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorepasswd>

List of accounts to ignore if they have no password.  The C<sync> account
in SunOS 4.x is an example.

=item C<ignoreuid>

List of accounts to ignore that share uid 0.  C<root> and C<toor> accounts
on FreeBSD are examples.

=item C<ignoredupuid>

List of accounts to ignore that share NON 0 uid's.

=item C<ignoreduppasswd>

List of accounts to ignore that share a password.

=item C<defaultpass>

List of password hashes to use as the list of known passwords.

=back

=head1 HISTORY

The idea of looking for accounts without a password was suggested in
the BSD 4.4 security script.

The idea of looking for duplicate UID 0 accounts was suggested in the
BSD 4.4 security script.

This idea of looking for duplicate non UID 0 accounts was suggested
in the C<HP-UX System Security Manual>

=head1 SEE ALSO

L<Security::util::users>

=cut

