#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::rootlogin - root login control module
#
package Security::rootlogin;

use Exporter;
use strict;
use warnings;
no warnings 'once';

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check_ttys() &check_access() &check_defaultlogin() &print() );

our (%ttys);

sub init {
	print ( STDERR "In Security::rootlogin::init()\n") if ( $main::opts{'d'} );

	# Read the config file for root port definitions.
	&main::read_config('ignorerootports');

	$main::rootlogin = -1;
}

sub check_securettys {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_securettys()\n") if ( $main::opts{'d'} );

	open (FP, $main::securefile) || 
		&main::security_die ("Could not open login file $main::securefile\n");
	grep (/^(.*)$/o && ($ttys{$1}=1), <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_defaultlogin {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_defaultlogin()\n") if ( $main::opts{'d'} );
	open (FP, $main::loginfile) || &main::security_die ("Could not open login file $main::loginfile\n");
	$main::rootlogin=grep (/^CONSOLE/o, <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_ttys {
	local *FP;

	printf STDERR "In Security::rootlogin::check_ttys()\n"
		if ( $main::opts{'d'} );
	open (FP, $main::securefile) || 
		&main::security_die ("Could not open login file $main::securefile\n");
	grep (/^([^ \t#]+)[ \t]*.*[ \t]*on[ \t]*secure/o && ($ttys{$1}=1), <FP>);
	close FP;

	&_remove_defaultports();
}

sub check_access {
	local *FP;

	print ( STDERR "In Security::rootlogin::check_access()\n") if ( $main::opts{'d'} );
	open (FP, $main::accessfile) || 
		&main::security_die ("Could not open login file $main::accessfile\n");
	while (<FP>) {
		my (@a) = split (/:/o, $_) if ( $_ !~ /^#/o );
		if ( $a[1] eq "root" ) {
			if ( $a[0] eq '+' ) {
				if ( $a[2] =~ /ALL$/o ) {
					$main::rootlogin=0;
				} else {
					my (@b) = split (/[ \t]+/o, $a[2]);
					for (my ($i) = 0; $i < $#b; $i++) {
						$ttys{$b[$i]} = 1 if ( $b[$i] ne 'ALL' );
					}
				}
			} else {
				if ( $a[2] =~ /ALL$/o ) {
					$main::rootlogin=1;
				} else {
					my (@b) = split (/[ \t]+/o, $a[2]);
					for (my ($i) = 0; $i < $#b; $i++) {
						$ttys{$b[$i]} = 1 
							if ( $b[$i] ne 'ALL' && 
								$b[$i] ne 'EXCEPT' );
					}
				}
			}
		}
	}
	close FP;

	&_remove_defaultports();
}

sub _remove_defaultports {
	# Now remove the preconfigured root ports
	for (my ($i) = 0; $i < scalar (@main::ignorerootports); $i++) {
		if ( defined $ttys{$main::ignorerootports[$i]} ) {
			printf STDERR "unsetting $main::ignorerootports[$i] in ttys array\n"
				if ( $main::opts{'d'} );
			delete $ttys{$main::ignorerootports[$i]};
		}
	}
}

sub print {
	printf STDERR "In Security::rootlogin::print()\n" if ( $main::opts{'d'} );

	if ( ! $main::rootlogin ) {
		&main::print_header("\n\nRoot login allowed on all ports\n");
     }

	&main::print_data ("\n\nchecking for ports root is allowed to login to:\n", %ttys);
}

1;

__END__

=head1 NAME

Security::rootlogin - root login control module

=head1 SYNOPSIS

Security::rootlogin::init();

Security::rootlogin::print();

=head1 DESCRIPTION

Verifies that root is allowed to login on only certain ports.

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (ignorerootports)> to get the list of ports that root
is allowed to login to.

=item C<print()>

Calls C<print_data ()> to print out the list of accounts root can login on
that we do not know about.

=back

It also provides the following API calls for the OS specific modules
to use.

=over 4

=item C<check_securettys>

For OSes that have F</etc/securetty> file to control root console ports. 

=item C<check_defaultlogin>

For OSes that have F</etc/default/login> file to control root console ports. 

=item C<check_ttys>

For OSes that have F</etc/ttys> file to control root console ports. 

=item C<check_access>

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<ignorerootports>

The list of ports that root is allowed to login on.

=back

=head1 TODO

Document check_access and find an OS it is used on.

=cut

