#
#  Copyright 1996-2008 Jim Pirzyk / pirzyk.org
#  All rights reserved.
#
#  Security::sudo - sudo control module
#
package Security::sudo;

use Security::db;
use Security::util::stat;
use Security::util::users;
use Exporter;
use POSIX;
use strict;
use warnings;

our @ISA = qw(Exporter);
our @EXPORT_OK = qw( &init() &check() &print() );

our (%sudoworld, %sudogroup, %sudoowner, %sudomissing, %groups);

sub init {
	print ( STDERR  "In Security::sudo::init()\n") if ( $main::opts{'d'} );

	&main::read_config('nosudoowner');
}

sub _get_group_members () {
	my ($gid) = @_;
	my ($nm, $pw, $u, $g);

	my (@gr) = getgrgid($gid);
	map { $groups{$_} = 1; } split /\s/o, $gr[3];

	# Now find the accounts that have this group as the primary gid
	foreach my $name ( Security::util::users::get('name')) {
		my $g = Security::util::users::get('name', $name, 'gid');
		$groups{$name} = 1
			if ($g eq $gid);
	}

	return keys %groups;
}

sub check {
	print ( STDERR  "In Security::sudo::check()\n") if ( $main::opts{'d'} );

	my ($currsum, $cmds, $cmd, $who);
	open (FP, $main::sudofile ) || &main::security_die ("Could not read sudoers file ($main::sudofile)\n");
	while (<FP>) {
		chomp;
		if ( ! /^Default/o ) {
			my (@a) = split;

			# Now find the command.
			if ( exists $a[1] && defined $a[1] ) {
				grep (/:(.*)$/o && ($cmd=$1), $a[1]);
				grep (/=\((.*)\)/o && ($who=$1), $a[1]);

				# Now expand any wildcarding in the $cmd
				if ( defined $cmd && length $cmd && defined $who && length $who ) {
					foreach my $cmd2 ( glob $cmd ) {
						push @{$cmds->{$cmd2}->{who}}, $who;
					}
				}
			}
		}
	}
	close FP;

	# Clean up the who list to have only uniq names
	foreach $cmd ( keys %$cmds ) {
		my (%a);
		map { $a{$_} = 1 } @{$cmds->{$cmd}->{who}};
		@{$cmds->{$cmd}->{who}} = sort keys %a;
	}

	# Now get mode/uid/gid info for each program 
	# and check who can write to the file.
	foreach $cmd ( keys %$cmds ) {
 		if ( ! -f $cmd ) {
			$sudomissing{$cmd} = 1;		
 		} else {
			&Security::db::verify($cmd);

			my ($dev,$ino,$mode,$nlink,$uid,$gid,$rdev,$curbytes) = 
				&Security::util::stat::get_stat($cmd, 'stat');

			# If the file is world writeable, then flag it
			$sudoworld{$cmd} = 1
				if ( $mode & &S_IWOTH() );

			# Now check if it is group writeable, and who is in the group
			if ( $mode & &S_IWGRP() ) {
				my @list = &_get_group_members ($gid);
				$who = Security::util::users::get('uid', $uid, 'name');

				# If the owner is NOT in the list of group members, then
				# complain about this.
				if (!grep (/^$who$/, @list)) {
					$sudogroup{$cmd} = 1;
				}
			}

			# Lastly, check to make sure if the file is user writeable
			# and who owns the file (exclude files owned by root or
			# bin (uid=$binUID).  This can be ignore if nosudoowner is
			# set in the config file.
			if ( ${main::nosudoowner[0]} ne 'yes' && $mode & &S_IWUSR() && $uid != 0 && $uid != $main::binuid ) {
				$who = Security::util::users::get('uid', $uid, 'name');
				if (! grep (/^${who}$/, @{$cmds->{$cmd}->{who}})) {
					$sudoowner{$cmd} = 1;
				}
			}
		}
	}
}

sub print {
	print ( STDERR  "In Security::sudo::print()\n") if ( $main::opts{'d'} );

	&main::print_data ("\n\nchecking sudo missing commands:\n", %sudomissing);
	&main::print_data ("\n\nchecking for world writable sudo commands:\n", %sudoworld);
	&main::print_data ("\n\nchecking for group writable sudo commands:\n", %sudogroup);
	&main::print_data ("\n\nchecking for owner mismatch sudo commands:\n", %sudoowner);
}

1;

__END__

=head1 NAME

Security::sudo - sudo control module

=head1 SYNOPSIS

Security::sudo::init()

Security::sudo::check()

Security::sudo::print()

=head1 DESCRIPTION

Check the owner, group and mode of all commands configured in the
F<sudoers> file.  The command should be owned by either C<root>, (C<bin>
on Solaris) or the account the command is going to be run as (usually this
is C<root> as well).

=head1 API

This module implements the default API:

=over 4

=item C<init()>

Calls C<read_config (nosudoowner)> to see if we do not want to test file
ownership.

=item C<check()>

Parses the F<sudoers> file, looking for all commands that are configured
and who the command is going to be run as.

=item C<print()>

Calls C<print_data ()> to print out the list of commands in the F<sudoers>
file but do not exist on the system.

Calls C<print_data ()> to print out the list of commands in the F<sudoers>
file that are world writable.

Calls C<print_data ()> to print out the list of commands in the F<sudoers>
file that are group writable.

Calls C<print_data ()> to print out the list of commands in the F<sudoers>
file that are not owned by the account they are being run as.

=back

=head1 CONFIGURATION OPTIONS

The script will call C<read_config> on the following config file options:

=over 4

=item C<nosudoowner>

Turn off the test for ownership of sudo commands.

=back

=head1 HISTORY

the C<nosudoowner> option was suggested by a co-worker at UIUC, for scripts
that were not owned by who they were being executed as.

=head1 SEE ALSO

L<Security::db>

L<Security::util::stat>

L<Security::util::users>

L<sudo(8)>

=cut

